SystemOrganization addCategory: #OrionGenerator!
SystemOrganization addCategory: #'OrionGenerator-Test'!

TestCase subclass: #ASGeneratorTest
	instanceVariableNames: 'aGenerator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OrionGenerator-Test'!

!ASGeneratorTest methodsFor: 'running' stamp: 'AnneEtien 4/3/2013 17:09'!
setUp
	
	aGenerator := ASGenerator new.! !

!ASGeneratorTest methodsFor: 'tests' stamp: 'AnneEtien 4/8/2013 13:20'!
testAddAccessorFrom
	
	|  aTrait nbTraits nbSel nbProp |

	nbTraits := Smalltalk allTraits size.

	aTrait := aGenerator ensureASTrait: FAMIXTestClass .
	nbSel := aTrait selectors size.
	nbProp := (aGenerator getPropertiesOf: FAMIXTestClass) size.
	
	"check the creation of the trait"
	self assert: aTrait notNil.
	self assert: (aTrait name) equals: #TFAMIXTestClass.
	self assert: Smalltalk allTraits size equals: (nbTraits + 1).
	self assert: aTrait selectors size equals: TMooseEntity selectors size.
	self assert: (aTrait traitComposition allTraits includes: TMooseEntity).
	
	aGenerator addAccessorFrom: FAMIXTestClass toTrait: aTrait. 
	self assert: aTrait selectors size equals:  (nbSel + (2*nbProp)).
	self assert: (aTrait selectors includes: #kind).
	self assert: (aTrait selectors includes: #kindAS:).
	"self assert: (aTrait selectors includes: #test).
	self assert: (aTrait selectors includes: #testAS:)."
	
	aTrait removeFromSystem.
	self assert: Smalltalk allTraits size equals: nbTraits.
	self deny: (Smalltalk allTraits includes: aTrait).! !

!ASGeneratorTest methodsFor: 'tests' stamp: 'AnneEtien 4/8/2013 13:04'!
testAddPrivateAccessorTo
	
	| nbMeth |

	
	nbMeth := FAMIXTestClass methodDict size.
	
	"self halt."
	aGenerator addPrivateAccessorTo: FAMIXTestClass. 
	self assert: (FAMIXTestClass methodDict size >= nbMeth ).
	self assert: (FAMIXTestClass methodDict includesKey:  #privateKind).
	self assert: (FAMIXTestClass methodDict includesKey: #privateKind:).
	self assert: ((FAMIXTestClass methodDict at: #privateKind) protocol) equals: '*OrionCore-model'. 
	
	! !

!ASGeneratorTest methodsFor: 'tests' stamp: 'AnneEtien 4/5/2013 18:51'!
testAddThePrivateAccessorOf
	
	| nbMeth |

	
	nbMeth := FAMIXTestClass methodDict size.
	
	
	aGenerator addThePrivateAccessorOf: (aGenerator getPropertiesOf: FAMIXTestClass) first To: FAMIXTestClass. 
	self assert: (FAMIXTestClass methodDict size =  (nbMeth + 2) or: [FAMIXTestClass methodDict size =  nbMeth] ).
	self assert: (FAMIXTestClass methodDict includesKey:  #privateKind).
	self assert: (FAMIXTestClass methodDict includesKey: #privateKind:).

	"self assert: ((FAMIXTestClass methodDict at: #privateKind) protocol) equals: '*OrionCore-model'. 
	"
	! !

!ASGeneratorTest methodsFor: 'tests' stamp: 'AnneEtien 4/3/2013 18:02'!
testCodeForASSelector

	self assert: (aGenerator codeForASSelector: (aGenerator getPropertiesOf: FAMIXTestClass) first) equals: 'kindAS: aVersion
	^ self privateKind versionIn: aVersion'! !

!ASGeneratorTest methodsFor: 'tests' stamp: 'AnneEtien 4/8/2013 12:56'!
testCodeForPrivateGetter

	self assert: (aGenerator codeForPrivateGetter: (aGenerator getPropertiesOf: FAMIXTestClass ) first) equals: 'privateKind
	^ kind'! !

!ASGeneratorTest methodsFor: 'tests' stamp: 'AnneEtien 4/8/2013 12:55'!
testCodeForPrivateSetter

	self assert: (aGenerator codeForPrivateSetter: (aGenerator getPropertiesOf: FAMIXTestClass ) first) equals: 'privateKind: aString
	 kind := aString'! !

!ASGeneratorTest methodsFor: 'tests' stamp: 'AnneEtien 4/8/2013 12:54'!
testCodeForSelector

	self assert: (aGenerator codeForSelector: (aGenerator getPropertiesOf: FAMIXTestClass ) first) equals: 'kind
	^ self kindAS: self currentContext'! !

!ASGeneratorTest methodsFor: 'tests' stamp: 'AnneEtien 4/3/2013 17:04'!
testCreateClassName

	self assert: (aGenerator createClassName: FAMIXPackage ) equals: #ASPackage.
	! !

!ASGeneratorTest methodsFor: 'tests' stamp: 'AnneEtien 4/3/2013 17:04'!
testCreateClassNameNotFamix

	self assert: (aGenerator createClassName: MooseEntity ) equals: #ASMooseEntity.
	! !

!ASGeneratorTest methodsFor: 'tests' stamp: 'AnneEtien 4/3/2013 17:04'!
testCreateTraitName

	self assert: (aGenerator createTraitName: FAMIXPackage ) equals: #TFAMIXPackage.
	! !

!ASGeneratorTest methodsFor: 'tests' stamp: 'AnneEtien 4/3/2013 17:04'!
testCreateTraitNameNotFamix

	self assert: (aGenerator createTraitName: MooseEntity ) equals: #TMooseEntity.
	! !

!ASGeneratorTest methodsFor: 'tests' stamp: 'AnneEtien 4/8/2013 14:00'!
testEnsureASClass
	
	|  anASClass nbClass |
	
	nbClass := Smalltalk allClasses size.
	
	anASClass := aGenerator ensureASClass: FAMIXTestClass.

	self assert: anASClass notNil.
	self assert: (anASClass name) equals: #ASTestClass.
	self assert: (anASClass traitComposition allTraits contains: [:each | each name = #TFAMIXTestClass]).
	self assert: (anASClass superclass) equals: FAMIXTestClass.
	self assert: Smalltalk allClasses size equals: nbClass + 1.
	
	anASClass removeFromSystem.
	(Smalltalk globals at: #TFAMIXTestClass) removeFromSystem.! !

!ASGeneratorTest methodsFor: 'tests' stamp: 'AnneEtien 4/8/2013 14:00'!
testEnsureASTrait
	
	|   aTrait nbTraits |

	nbTraits := Smalltalk allTraits size.

	aTrait := aGenerator ensureASTrait: FAMIXTestClass .
	
	self assert: aTrait notNil.
	self assert: (aTrait name) equals: #TFAMIXTestClass.
	self assert: Smalltalk allTraits size equals: (nbTraits + 1).
	self assert: aTrait selectors size equals: TMooseEntity selectors size.
	self assert: (aTrait traitComposition allTraits includes: TMooseEntity).
	
	aTrait removeFromSystem.! !

!ASGeneratorTest methodsFor: 'tests' stamp: 'AnneEtien 4/8/2013 14:09'!
testEnsureASTraitExisting
	
	|   aTrait nbMeth nbTraits |

	nbTraits := Trait allInstances size.
	nbMeth := TFAMIXClass selectors size.

	aTrait := aGenerator ensureASTrait: FAMIXClass .

	self assert: aTrait notNil.
	self assert: (aTrait name) equals: #TFAMIXClass.
	self assert: Trait allInstances size equals: nbTraits.
	self assert: aTrait selectors size equals: nbMeth.
	! !

!ASGeneratorTest methodsFor: 'tests' stamp: 'AnneEtien 4/11/2013 15:09'!
testEnsureASTraitWithoutExistingTraits
	
	|   aTrait nbTraits |

	nbTraits := Smalltalk allTraits size.

	aTrait := aGenerator ensureASTrait: Class1 .
	
	self assert: aTrait notNil.
	self assert: (aTrait name) equals: #TClass1.
	self assert: Smalltalk allTraits size = (nbTraits + 4).
	"self assert: aTrait selectors size equals: TMooseEntity selectors size.
	self assert: (aTrait traitComposition allTraits includes: TMooseEntity)."
	
	aTrait removeFromSystem.
	(Smalltalk globals at: #TFAMIXLocalVariable) removeFromSystem.
	(Smalltalk globals at: #TFAMIXLeafEntity) removeFromSystem.
	(Smalltalk globals at: #TFAMIXStructuralEntity) removeFromSystem.
	
	self assert: Smalltalk allTraits size = nbTraits ! !

!ASGeneratorTest methodsFor: 'tests' stamp: 'AnneEtien 4/8/2013 12:48'!
testGetPropertiesOf

	self assert: (aGenerator getPropertiesOf: FAMIXTestClass ) size  equals: 1
! !

!ASGeneratorTest methodsFor: 'tests' stamp: 'AnneEtien 4/3/2013 17:51'!
testGetPropertiesOfEmpty

	self assert: (aGenerator getPropertiesOf: Object) size  equals: 0
! !

!ASGeneratorTest methodsFor: 'tests' stamp: 'AnneEtien 4/8/2013 14:28'!
testMakeAS

	|   nbTraits anASClass nbClass nbMeth |

	nbTraits := Smalltalk allTraits size.
	nbClass := Smalltalk allClasses size.
	nbMeth := FAMIXTestClass methodDict size.
	
	aGenerator makeAS: FAMIXTestClass .
	anASClass := Smalltalk classNamed: #ASTestClass.
	self assert: (Smalltalk allTraits contains: [:each | each name = #TFAMIXTestClass]).
	self assert: Smalltalk allTraits size equals: (nbTraits + 1).
	self assert: (Smalltalk allClasses contains: [:each | each name = #ASTestClass]).
	self assert: (anASClass traitComposition allTraits contains: [:each | each name = #TFAMIXTestClass]).
	self assert: (anASClass superclass) equals: FAMIXTestClass.
	self assert: Smalltalk allClasses size equals: nbClass + 1.
	
	self assert: (FAMIXTestClass methodDict size >= nbMeth ).
	self assert: (FAMIXTestClass methodDict includesKey:  #privateKind).
	self assert: (FAMIXTestClass methodDict includesKey: #privateKind:).
	self assert: ((FAMIXTestClass methodDict at: #privateKind) protocol) equals: '*OrionCore-model'. 
	
	anASClass removeFromSystem.
	(Smalltalk globals at: #TFAMIXTestClass) removeFromSystem.! !

!ASGeneratorTest methodsFor: 'tests' stamp: 'AnneEtien 4/11/2013 15:41'!
testMakeASForAllClasses

	|   nbTraits nbClass nbMeth aColl |
	
	aColl := Smalltalk allClasses select: [ :c | (c category =#'Orion-TestModel-Constellation1')].	

	nbTraits := Smalltalk allTraits size.
	nbClass := Smalltalk allClasses size.
	nbMeth := FAMIXTestClass methodDict size.
	
	self deny: (Smalltalk allClasses contains: [:each | each name = #ASTestClass]).
	self deny: (Smalltalk allClasses contains: [:each | each name = #ASClass1]).
	self deny: (Smalltalk allClasses contains: [:each | each name = #ASClass2]).

	self deny: (Smalltalk allTraits contains: [:each | each name = #TFAMIXTestClass]).
	self deny: (Smalltalk allTraits contains: [:each | each name = #TClass1]).
	self deny: (Smalltalk allTraits contains: [:each | each name = #TClass2]).
	
	aGenerator makeASForAllClasses: aColl .
	
	"verification that the 3 traits have been created"
	self assert: (Smalltalk allTraits contains: [:each | each name = #TFAMIXTestClass]).
	self assert: (Smalltalk allTraits contains: [:each | each name = #TClass1]).
	self assert: (Smalltalk allTraits contains: [:each | each name = #TClass2]).
	self assert: Smalltalk allTraits size equals: (nbTraits + 3+ aColl size). "+3 corresponds to the hierarchy of Traits added for TClass1"
	
	
	"Removing the traits and the classes created during the test"
	(Smalltalk globals at: #TFAMIXTestClass) removeFromSystem.
	(Smalltalk globals at: #TClass1) removeFromSystem.
	(Smalltalk globals at: #TClass2) removeFromSystem.
	(Smalltalk globals at: #TFAMIXLocalVariable) removeFromSystem.
	(Smalltalk globals at: #TFAMIXLeafEntity) removeFromSystem.
	(Smalltalk globals at: #TFAMIXStructuralEntity) removeFromSystem.
	(Smalltalk globals at: #ASTestClass) removeFromSystem.
	(Smalltalk globals at: #ASClass1) removeFromSystem.
	(Smalltalk globals at: #ASClass2) removeFromSystem.
	
	self assert: Smalltalk allTraits size equals: nbTraits.
	self assert: Smalltalk allClasses size equals: nbClass .! !

Object subclass: #ASGenerator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OrionGenerator'!

!ASGenerator methodsFor: 'as yet unclassified' stamp: 'AnneEtien 4/3/2013 17:18'!
addAccessorFrom: aFamixClass toTrait: aOrionTrait
	"add accessors corresponding to the non heritated properties and relations of the Famix Class to the Orion trait "
	
	(self getPropertiesOf: aFamixClass) do: [ :prop | 
		aOrionTrait compile: (self codeForSelector: prop).
		aOrionTrait compile: (self codeForASSelector: prop).
		]
	! !

!ASGenerator methodsFor: 'as yet unclassified' stamp: 'AnneEtien 4/8/2013 13:12'!
addPrivateAccessorTo: aFamixClass
	"add private accessors corresponding to the non heritated properties and relations of the Famix Class to it "
	(self getPropertiesOf: aFamixClass) do: [ :prop | 
		self addThePrivateAccessorOf: prop To: aFamixClass
		]
	
	! !

!ASGenerator methodsFor: 'as yet unclassified' stamp: 'AnneEtien 4/8/2013 13:12'!
addThePrivateAccessorOf: aProp To: aFamixClass 
	"add private accessors corresponding to the property to the Famix Class if it does not already exist "
	 	| aMethName |
		aMethName := 'private', (aProp name) capitalized.
		(aFamixClass methodDict includesKey: (aMethName asSymbol)) ifFalse: 
			[aFamixClass compile: (self codeForPrivateGetter: aProp) classified: '*OrionCore-model'.
			aFamixClass compile: (self codeForPrivateSetter: aProp) classified: '*OrionCore-model'.]
	! !

!ASGenerator methodsFor: 'as yet unclassified' stamp: 'AnneEtien 4/3/2013 17:29'!
codeForASSelector: aProperty
	^((aProperty name), 'AS: aVersion
	^ self private' , (aProperty name) capitalized , ' versionIn: aVersion')
	! !

!ASGenerator methodsFor: 'as yet unclassified' stamp: 'AnneEtien 4/5/2013 18:19'!
codeForPrivateGetter: aProperty
	^('private', (aProperty name) capitalized, '
	^ ' ,  (aProperty name))
	! !

!ASGenerator methodsFor: 'as yet unclassified' stamp: 'AnneEtien 4/5/2013 18:21'!
codeForPrivateSetter: aProperty
	| type|
	type := ((aProperty type name) asString ) capitalized.
	^('private', (aProperty name) capitalized, ': a' , type , '
	 ' ,  (aProperty name), ' := a' ,  type)
	! !

!ASGenerator methodsFor: 'as yet unclassified' stamp: 'AnneEtien 4/3/2013 17:20'!
codeForSelector: aProperty
	^((aProperty name), '
	^ self ' , (aProperty name), 'AS: self currentContext')
	! !

!ASGenerator methodsFor: 'as yet unclassified' stamp: 'AnneEtien 4/3/2013 13:14'!
createClassName: aFamixClass
	"Create the name of the Orion Class from the FamixClass"

	| name |

	name := aFamixClass name asString.
	name := (name beginsWith: 'FAMIX') ifTrue:  [name copyAfter: $X]
					ifFalse: [name].
	^ ('AS' , name ) asSymbol ! !

!ASGenerator methodsFor: 'as yet unclassified' stamp: 'AnneEtien 4/3/2013 13:16'!
createTraitName: aFamixClass
	"Create the name of the Orion Class from the FamixClass"

	| name |

	name := aFamixClass name asString.
	^ ('T' , name ) asSymbol ! !

!ASGenerator methodsFor: 'as yet unclassified' stamp: 'AnneEtien 4/8/2013 13:19'!
ensureASClass: aFamixClass
	"Create an Orion Class corresponding to the FamixClass and inheriting from it"

	| name |
	name := self createClassName: aFamixClass .
	
	^Smalltalk globals at: name ifAbsent: [
		aFamixClass  subclass: name
		uses: (self ensureASTrait: aFamixClass)
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'OrionCore-model']! !

!ASGenerator methodsFor: 'as yet unclassified' stamp: 'AnneEtien 4/11/2013 14:53'!
ensureASTrait: aFamixClass
	"Create a trait corresponding to the FamixClass "

	^ aFamixClass name = #MooseEntity
		ifFalse: [ 
			| name |
			name := self createTraitName: aFamixClass.
			Smalltalk globals
				at: name
				ifAbsent: [ Trait
								named: name
								uses: (self ensureASTrait: aFamixClass superclass)
								category: 'OrionCore-model-trait' ] ]
		ifTrue: [ TMooseEntity ]! !

!ASGenerator methodsFor: 'as yet unclassified' stamp: 'AnneEtien 4/8/2013 12:53'!
getPropertiesOf: aFamixClass

	| metaDesc |
	metaDesc := MooseModel meta elements detect: [ :md | (md class = FM3MetaDescription  ) and: [md implementingClass = aFamixClass  ] ] ifNone: [ nil ].
	
	^ (metaDesc notNil)
	ifTrue: [ metaDesc attributes select: [:a | a isDerived not and: [aFamixClass allInstVarNames includes: (a name) ]] ] ifFalse: [ OrderedCollection new ]! !

!ASGenerator methodsFor: 'as yet unclassified' stamp: 'AnneEtien 4/11/2013 15:23'!
makeAS: aFamixClass
	"Create the ASClass corresponding to the FamixClass, add accessors to the trait and accessors to the FamixClass"

	self addAccessorFrom: aFamixClass toTrait: (self ensureASTrait: aFamixClass).
	self ensureASClass: aFamixClass.
	self addPrivateAccessorTo: aFamixClass! !

!ASGenerator methodsFor: 'as yet unclassified' stamp: 'AnneEtien 4/8/2013 15:19'!
makeASForAllClasses: aColl
	"For all the class of the collection, Create the ASClass corresponding to the FamixClass, add accessors to the trait and accessors to the FamixClass"
	
	aColl do: [ :c | self makeAS: c ]! !
